div.bodywrapper {
    border-left: solid #ddd;
}

div.related {
    background-color: #006c85;
}

div.related a {
    color: #7CB9B3    
}

div.document {
    background-color: #fff;
}

div.sphinxsidebar p {
    color: #006c85;
}

div.body h1 {
    background-color: #C7E3E2;
	font-size: 120%;
	padding: 2px 0 2px 5px;
}
    
div.body h2 {
    background-color: #C7E3E2;
	font-size: 100%;
	padding: 2px 2px 2px 5px;
}

div.admonition {
	padding: 2px 2px 2px 5px;        
}

div.seealso {
    background-color: #eafaff;
    border: 1px solid #eafaff;
}

dt:target, span.highlighted {
  background-color: #e9fce6;
}

.center {text-align: center;}

img.logo {
  display:block;
  margin:0 auto;
  height:auto;
  width:auto;
  border-radius:0;
  max-width:100%;
  background:transparent
}


div.figure.align-default img {
  display: block;
  width:auto;
  border-radius:0;
  max-width:100%;
  background:transparent;
  margin-left: auto;
  margin-right: auto;
}

div.figure img {
  display: block;
  width:auto;
  border-radius:0;
  max-width:100%;
  background:transparent;
  margin-left: auto;
  margin-right: auto;
}